@@ -12,6 +12,7 @@ import sqlite3
 import shutil
 import datetime as dt
 
+from .dbproto import StructuredMentions
 from .dbproto import StructuredReaction
 from .dbproto import StructuredReactions
 
@@ -22,6 +23,7 @@ from .exceptions import DatabaseVersionNotFound
 
 from .models import Attachment
 from .models import MMSMessageRecord
+from .models import Mention
 from .models import Quote
 from .models import Reaction
 from .models import Recipient
@@ -128,7 +130,38 @@ def add_mms_attachments(db, mms, backup_dir, thread_dir):
         mms.attachments.append(a)
 
 
+def get_mms_mentions(encoded_mentions, addressbook, mid):
+    """Decode mentions encoded in a SQL blob."""
+    mentions = {}
+    if encoded_mentions:
+        try:
+            structured_mentions = StructuredMentions.loads(encoded_mentions)
+        except (ValueError, IndexError, TypeError) as e:
+            logger.warn(
+                f"Failed to load quote mentions for message {mid}: {str(e)}"
+            )
+            return []
+
+        for structured_mention in structured_mentions.mentions:
+            recipient = addressbook.get_recipient_by_uuid(
+                structured_mention.who_uuid
+            )
+            name = recipient.name
+            mention = Mention(
+                mention_id=-1, name=name, length=structured_mention.length
+            )
+            range_start = (
+                0
+                if structured_mention.start is None
+                else structured_mention.start
+            )
+            mentions[range_start] = mention
+
+    return mentions
+
+
 def get_mms_reactions(encoded_reactions, addressbook, mid):
+    """Decode reactions encoded in a SQL blob."""
     reactions = []
     if encoded_reactions:
         try:
@@ -172,9 +205,11 @@ def get_mms_records(
 
     reaction_expr = versioninfo.get_reactions_query_column()
 
+    quote_mentions_expr = versioninfo.get_quote_mentions_query_column()
+
     qry = db.execute(
         "SELECT _id, address, date, date_received, body, quote_id, "
-        f"quote_author, quote_body, msg_box, {reaction_expr} FROM mms WHERE thread_id=?",
+        f"quote_author, quote_body, {quote_mentions_expr}, msg_box, {reaction_expr} FROM mms WHERE thread_id=?",
         (thread._id,),
     )
     qry_res = qry.fetchall()
@@ -187,10 +222,18 @@ def get_mms_records(
         quote_id,
         quote_author,
         quote_body,
+        quote_mentions,
         msg_box,
         reactions,
     ) in qry_res:
-        quote = get_mms_quote(addressbook, quote_id, quote_author, quote_body)
+        quote = get_mms_quote(
+            addressbook,
+            quote_id,
+            quote_author,
+            quote_body,
+            quote_mentions,
+            _id,
+        )
 
         decoded_reactions = get_mms_reactions(reactions, addressbook, _id)
 
@@ -216,14 +259,55 @@ def get_mms_records(
     return mms_records
 
 
-def get_mms_quote(addressbook, quote_id, quote_author, quote_body):
+def get_mms_quote(
+    addressbook, quote_id, quote_author, quote_body, quote_mentions, mid
+):
+    """Retrieve quote (replied message) from a MMS message."""
     quote = None
     if quote_id:
         quote_auth = addressbook.get_recipient_by_address(quote_author)
-        quote = Quote(_id=quote_id, author=quote_auth, text=quote_body)
+        decoded_mentions = get_mms_mentions(quote_mentions, addressbook, mid)
+        quote = Quote(
+            _id=quote_id,
+            author=quote_auth,
+            text=quote_body,
+            mentions=decoded_mentions,
+        )
     return quote
 
 
+def get_mentions(db, addressbook, thread_id, versioninfo):
+    """Retrieve all mentions in the DB for the requested thread into a dictionary."""
+    mentions = {}
+
+    if versioninfo.are_mentions_supported():
+        query = db.execute(
+            "SELECT _id, message_id, recipient_id, range_start, range_length "
+            "FROM mention WHERE thread_id=?",
+            (thread_id,),
+        )
+        mentions_data = query.fetchall()
+
+        for (
+            _id,
+            message_id,
+            recipient_id,
+            range_start,
+            range_length,
+        ) in mentions_data:
+            name = addressbook.get_recipient_by_address(str(recipient_id)).name
+            mention = Mention(
+                mention_id=_id,
+                name=name,
+                length=range_length,
+            )
+            if not message_id in mentions.keys():
+                mentions[message_id] = {}
+            mentions[message_id][range_start] = mention
+
+    return mentions
+
+
 def populate_thread(
     db, thread, addressbook, backup_dir, thread_dir, versioninfo=None
 ):
@@ -240,6 +324,8 @@ def populate_thread(
     thread.sms = sms_records
     thread.mms = mms_records
 
+    thread.mentions = get_mentions(db, addressbook, thread._id, versioninfo)
+
 
 def process_backup(backup_dir, output_dir):
     """ Main functionality to convert database into HTML """
@@ -264,10 +350,10 @@ def process_backup(backup_dir, output_dir):
             logger.warn(f"No recipient with address {recipient_id}")
 
         t = Thread(_id=_id, recipient=recipient)
-        thread_dir = os.path.join(output_dir, t.sanename)
+        thread_dir = t.get_thread_dir(output_dir, make_dir=False)
         populate_thread(
             db, t, addressbook, backup_dir, thread_dir, versioninfo=versioninfo
         )
-        dump_thread(t, thread_dir)
+        dump_thread(t, output_dir)
 
     db.close()
