@@ -11,7 +11,7 @@ License: See LICENSE file.
 from dataclasses import dataclass
 from typing import List, Optional
 from pure_protobuf.dataclasses_ import field, message, optional_field
-from pure_protobuf.types import uint64
+from pure_protobuf.types import uint32, uint64
 
 
 @message
@@ -27,3 +27,17 @@ class StructuredReaction:
 @dataclass
 class StructuredReactions:
     reactions: List[StructuredReaction] = field(1, default_factory=list)
+
+
+@message
+@dataclass
+class StructuredMention:
+    start: uint32 = optional_field(1)
+    length: uint32 = optional_field(2)
+    who_uuid: str = optional_field(3)
+
+
+@message
+@dataclass
+class StructuredMentions:
+    mentions: List[StructuredMention] = field(1, default_factory=list)
