@@ -10,9 +10,11 @@ License: See LICENSE file.
 
 """
 
+import os
+
 from abc import ABCMeta
 from dataclasses import dataclass, field
-from typing import List
+from typing import List, Dict
 from re import sub
 from unicodedata import normalize
 from datetime import datetime
@@ -25,16 +27,25 @@ class Recipient:
     color: str
     isgroup: bool
     phone: str
+    uuid: str
 
     def __hash__(self):
         return hash(self.rid)
 
 
+@dataclass
+class Mention:
+    mention_id: int
+    name: str
+    length: int
+
+
 @dataclass
 class Quote:
     _id: int
     author: Recipient
     text: str
+    mentions: Dict[int, Mention] = field(default_factory=lambda: {})
 
 
 @dataclass
@@ -90,6 +101,11 @@ class Thread:
     recipient: Recipient
     mms: List[MMSMessageRecord] = field(default_factory=lambda: [])
     sms: List[SMSMessageRecord] = field(default_factory=lambda: [])
+    mentions: Dict[int, Dict[int, Mention]] = field(default_factory=lambda: {})
+
+    @property
+    def is_group(self) -> bool:
+        return self.recipient.isgroup
 
     @property
     def sanephone(self):
@@ -120,4 +136,28 @@ class Thread:
         clean = normalize("NFKC", text.strip())
         clean = clean.lstrip(".#")
         clean = sub("[^]\\w!@#$%^&'`.=+{}~()[-]", "_", clean)
+        clean = clean.rstrip("_")
         return clean
+
+    def get_thread_dir(self, output_dir: str, make_dir=True) -> str:
+        return os.path.dirname(self.get_path(output_dir, make_dir=make_dir))
+
+    def get_path(self, output_dir: str, make_dir=True) -> str:
+        """Return a path for a thread and try to be clever about merging
+        contacts. Optionally create the contact directory."""
+        dirname = self.sanename
+        # Use phone number to distinguish threads from the same contact,
+        # except for groups, which do not have a phone number.
+        filename = f"{self.sanename if self.is_group else self.sanephone}.html"
+        path = os.path.join(output_dir, dirname, filename)
+        i = 2
+        while os.path.exists(path):
+            if self.is_group:
+                dirname = f"{self.sanename}_{i}"
+            else:
+                filename = f"{self.sanephone}_{i}.html"
+            path = os.path.join(output_dir, dirname, filename)
+            i += 1
+        if make_dir:
+            os.makedirs(os.path.dirname(path), exist_ok=True)
+        return path
